
package expressions;

/*
	Alarm.java
	Copyright (c) A. Clausing (achim.clausing@uni-muenster.de)

	Exceptions for the t.Languages interpreter
*/

@SuppressWarnings("serial")
public class Alarm extends Exception implements Expr {

	/*
		An alarm has a name and a content
		The stack captures the current evaluations
	*/
	private Symbol name;     // Error name
	private Expr content;    // Error content (e. g., error message)
	private EvalStack stack; // Evaluation stack at error creation time

	/*
		An alarm with a name and a content
	*/
	public Alarm(Symbol nm, Expr con) {
		name  = nm;
		content = con;
		stack = EvalStack.stack();
	}

	/*
		Convenience: content is a message string
	*/
	public Alarm(String name, String msg) {
		this(Symbol.forName(name), new StringExpr(msg));
	}

	/*
		Access methods
	*/
	public Symbol name()  { return name; }
	public Expr content() { return content; }
	public Expr stack()   { return stack; }

	/*
		Evaluation
	*/
	public Expr eval(Env env, boolean... tailPosition) {
		return this;
	}

	/*
		Runtime type
	*/
	public Expr type() {
		return Symbol.forName("Exception");
	}

	/*
		String representation
	*/
	public String toString() {
		String msg = content().toString();
		return "Error" + (msg.equals("") ? "" : ": ") + msg;
	}

	/*
		Returns true, if this is EOF
	*/
	public boolean isEof() {
		return name.equals(Symbol.forName("EOF"));
	}
}

//--------------------------------------------------------------------------------------------------
