
package expressions;

/*
	Bool.java
	Copyright (c) A. Clausing (achim.clausing@uni-muenster.de)

	A Bool is a boolean expression
*/

public enum Bool implements Expr {

	/*
		Constants
	*/
	TRUE(true), FALSE(false);

	/*
		The boolean value of the expression
	*/
	private boolean val;

	/*
		Constructor
	*/
	private Bool(boolean b) {
		val = b;
	}

	/*
		Evaluation
	*/	
	public Expr eval(Env env, boolean... tailPosition) {
		return this;
	}
	
	/*
		Runtime type
	*/
	public Expr type() {
		return Symbol.forName("Boolean");
	}

	/*
		String representation
		(enum-Methode would give capitals)
	*/
	public String toString() {
		return val ? "true" : "false";
	}

	/*
		valueOf(b) returns the t.Languages equivalent of boolean b
	*/
	public static Bool valueOf(boolean b) {
		return b ? TRUE : FALSE;
	}

	/*
		isTrue(e) returns true if and only if e is TRUE
	*/
	public static boolean isTrue(Expr e) {
		return e.equals(TRUE);
	}

	/*
		isFalse(e) returns true if and only if e is FALSE
	*/
	public static boolean isFalse(Expr e) {
		return e.equals(FALSE);
	}

	/*
		Check e for being a Bool and return its Java equivalent
	*/
	public static boolean checkBool(Expr e, Expr client) throws Alarm {
		if (e instanceof Bool) return isTrue((Bool) e);
		throw new Alarm("TYPE", client + " expects a boolean argument");
	}
}

//--------------------------------------------------------------------------------------------------
