
package expressions;

/*
	EvalStack.java
	Copyright (c) A. Clausing (achim.clausing@uni-muenster.de)

	EvalStack contains the list of all expressions
	who are currently being evaluated.
*/

import tanagra.Sys;

public class EvalStack extends List {

	/*
		The evaluation stack
	*/
	private final static EvalStack NIL = new EvalStack();
	private static EvalStack stack = NIL;

	/*
		Protocol mode
	*/	
	private static boolean protocol = false;

	/*
		Constructor for empty EvalStack
	*/
	private EvalStack() {
		super();
	}

	/*
		Constructor for nonempty EvalStack
	*/
	private EvalStack(Expr e, EvalStack es) {
		super(e, es);
	}

	/*
		Push
	*/
	public static void push(Expr e, Env env) {
		if (protocol) write(e, true);
		stack = new EvalStack(e, stack);
	}

	/*
		Writes a protocol line
	*/
	private static void write(Expr e, boolean push) {
		String indent = "\t";
		int depth = stack.length() + (push ? 0 : -1);
		for (int i = 0; i < depth; i++) indent += "| ";
		indent += push ? "\\  " : "/  ";
		Sys.println(indent + e);
	}

	/*
		Pop 
	*/
	public static void pop() {
		pop(protocol);
	}

	/*
		Pop, no protocol
	*/	
	public static void popNoProtocol() {
		pop(false);
	}

	/*
		Pop, with protocolling dependig on protocol 
		(The stack might be corrupted by using op[setcdr], hence the 'if')
	*/
	private static void pop(boolean protocol) {
		if (!stack.isEmpty()) {
			if (protocol) {
				Expr e = stack.head();
				write(e, false);
			}	
			stack = (EvalStack) stack.tail();
		}	
	}

	/*
		Access to the stack
	*/
	public static EvalStack stack() {
		return stack;
	}

	/*
		Put the stack into state 'state'
	*/
	public static void setState(EvalStack state) {
		stack = state;
	}

	/*
		Check if stack is empty
	*/
	public static boolean empty() {
		return stack.isEmpty();
	}

	/*
		Reset the stack
	*/
	public static void reset() {
		stack = NIL;
	}

	/*
		Toggle protocol mode
	*/
	public static boolean toggleProtocolMode() {
		return protocol = ! protocol;
	}
}

//--------------------------------------------------------------------------------------------------
