
package expressions;

/*
	LinkList.java
	Copyright (c) A. Clausing (achim.clausing@uni-muenster.de)

	Generic list class.
*/

public class LinkList<T> {

	private T head;  
	private LinkList<T> tail;
	protected int length;

	/*
		Creates an empty list
	*/
	public LinkList() {
		length = 0;
	}

	/*
		Creates a nonempty list
	*/
	public LinkList(T e, LinkList<T> ls) {
		head = e;
		tail = ls;
		length = 1 + ls.length;
	}

	/*
		Access to head
	*/
	public T head() {
		return head;
	}

	/*
		Access to tail
	*/
	public LinkList<T> tail() {
		return tail;
	}

	/*
		Access to length
	*/
	public int length() {
		return length;
	}

	/*
		Setter for head
	*/
	protected void setHead(T t) {
		head = t;
	}

	/*
		Setter for tail
	*/
	protected void setTail(LinkList<T> ls) {
		tail = ls;
	}

	/*
		Test for empty list
	*/
	public boolean isEmpty() {
		return length == 0;
	};

	/*
		Equality is defined elementwise
	*/
	public boolean equals(Object obj) {
		if (!(obj instanceof LinkList<?>)) return false;
		LinkList<?> ls = (LinkList<?>) obj;
		return (length == ls.length) &&
			(isEmpty() || (head.equals(ls.head) && tail.equals(ls.tail)));
	}

	/*
		Returns true if this List contains object obj
	*/
	public boolean contains(Object obj) {
		if (isEmpty()) return false;
		return head.equals(obj) || tail.contains(obj);
	}

	/*
		String form
	*/
	public String toString() {
		return toStringWith();
	}

	/*
		Returns a string form with parentheses
	*/
	protected String toStringWith() {
		return "(" + toStringWithout() + ")";
	}

	/*
		Returns a string form without parentheses
	*/
	protected String toStringWithout() {
		if (isEmpty()) {
			return "";
		} else if (tail.isEmpty()) {
			return head.toString();
		} else {
			return head.toString() + " " + tail.toStringWithout();
		}	
	}

	/*
		Returns the first element of this list
		Assumes length >= 1
	*/
	public T first() {
		return head;
	}

	/*
		Returns the second element of this list
		Assumes length >= 2
	*/
	public T second() {
		return tail.head;
	}

	/*
		Returns the third element of this list
		Assumes length >= 3
	*/
	public T third() {
		return tail.tail.head;
	}
}

//--------------------------------------------------------------------------------------------------
