
package expressions;

/*
	Procedure.java
	Copyright (c) A. Clausing (achim.clausing@uni-muenster.de)

	The abstract base class for all procedures.
*/

public abstract class Procedure implements Expr {

	/*
		Procedures can be applied to arguments
	*/
	public abstract Expr apply(List args, Env env, boolean tailPosition) throws Alarm;

	/*
		Check this procedure for correct number of arguments.
		n is the expected number of args.
	*/
	public void checkArity(List args, int n) throws Alarm {
		if (args.length() == n) return;
		throw new Alarm("ARITY", this + " expects " + n +
			" argument" + (n == 1 ? "" : "s"));
	}

	/*
		Check this procedure for correct number of arguments.
		The expected number of args is between a and b.
	*/
	public void checkArity(List args, int a, int b) throws Alarm {
		if (a <= args.length() && args.length() <= b) return;
		String to = ((a + 1) == b) ? " or " : " to ";
		throw new Alarm("ARITY", this + " expects " + a + to + b + " arguments");
	}

	/*
		Check this procedure for correct number of arguments.
		The expected number of args is >= min.
	*/
	public void checkAritymin(List args, int min) throws Alarm {
		if (args.length() >= min) return;
		String msg = this + " expects at least " + min +
					" argument" + ((min > 1) ? "s" : "");
		throw new Alarm("ARITY", msg);
	}

	/*
		Check this procedure for an odd number of arguments.
	*/
	public void checkArityodd(List args) throws Alarm {
		if (args.length() % 2 == 1) return;
		throw new Alarm("ARITY", this + " expects an odd number of arguments");
	}
}

//--------------------------------------------------------------------------------------------------
