
package expressions;

/*
	Stream.java
	Copyright (c) A. Clausing (achim.clausing@uni-muenster.de)

	A Stream is an infinite list
*/

public class Stream implements Expr {

	private Expr head;         // head of this stream
	private Expr tail;         // tail of this stream
	private boolean evaluated; // has the tail been evaluated?
	private Env env;           // context for evaluation of tail

	/*
		Constructor
	*/
	public Stream(Expr x, Expr s, Env context) {
		head = x;
		tail = s;
		evaluated = false;
		env  = context;
	}        

	/*
		Returns the cdr
	*/
	public Stream cdr() throws Alarm {
		if (!evaluated) {
			tail = checkStream(tail.eval(env));
			evaluated = true;
		}
		return (Stream) tail;
	}

	/*
		Returns the head
	*/
	public Expr car() {
		return head;
	}	

	/*
		String form without brackets, n elements shown
	*/
	private String withoutBrackets(int n) {
		if (n == 0) {
			return "...";
		} else {
			try {
				return head + " " + cdr().withoutBrackets(n - 1);
			} catch (Alarm alarm) {
				return "...";
			}
		}    
	}

	/*
		Evaluation
	*/
	public Expr eval(Env env, boolean... tailPosition) {
		return this;
	}

	/*
		Runtime type
	*/
	public Expr type() {
		return Symbol.forName("Stream");
	}

	/*
		String form
	*/
	private final static int N = 12;   // number of visible stream elements
	public String toString() {
		return "(" + withoutBrackets(N) + ")";
	}		

	/*
		Check e for being a stream
	*/
	public static Stream checkStream(Expr e) throws Alarm {
		if (e instanceof Stream) return (Stream) e;
		throw new Alarm("TYPE", e + " is not a stream");
	}
}

//--------------------------------------------------------------------------------------------------
