
package expressions;

/*
	StringExpr.java
	Copyright (c) A. Clausing (achim.clausing@uni-muenster.de)

	A StringExpr is a string expression
*/

public class StringExpr implements Expr {

	/*
		The string itself
	*/
	private String str;

	/*
		Constructor
	*/
	public StringExpr(String s) {
		str = s;
	}

	/*
		Runtime type
	*/
	public Expr type() {
		return Symbol.forName("String");
	}

	/*
		Equality is by string value
	*/
	public boolean equals(Object e) {
		if (!(e instanceof StringExpr)) return false;
		return str.equals(((StringExpr) e).str);
	}

	/*
		Evaluation
	*/
	public Expr eval(Env env, boolean... tailPosition) {
		return this;
	}

	/*
		String representation
	*/
	public String toString() {
		return str;
	}

	/*
		Check e for being a string expression
	*/
	public static StringExpr checkStringexpr(Expr e, Expr client) throws Alarm {
		if (e instanceof StringExpr) return (StringExpr) e;
		throw new Alarm("TYPE", client + " expects a string argument");
	}
}

//--------------------------------------------------------------------------------------------------
