
package expressions;

/*
	Symbol.java
	Copyright (c) A. Clausing (achim.clausing@uni-muenster.de)

	A Symbol is a symbolic expression
*/

import java.util.Hashtable;

public class Symbol implements Expr {

	/*
		The classwide symbol table
	*/
	private static Hashtable<String, Symbol> symtable =
							new Hashtable<String, Symbol>(200);

	/*
		The symbol name
	*/
	private String name;

	/*
		Creates a symbol with name s
	*/
	private Symbol(String s) {
		name = s;
	}

	/*
		Returns the symbol with the name "name"
	*/
	public static Symbol forName(String name) {
		Symbol sym = symtable.get(name);
		if (sym == null) {
			sym = new Symbol(name);
			symtable.put(name, sym);
		}
		return sym;
	}

	/*
		Returns the value bound to this symbol in env
		Throws an alarm, if this symbol is unbound
	*/
	public Expr eval(Env env, boolean... tailPosition) throws Alarm {
		Expr e = env.retrieve(this);
		checkNotnull(e);
		return e;
	}

	/*
		Check retrieved expression for not being null
	*/
	private void checkNotnull(Expr e) throws Alarm {
		if (e != null) return;
		throw new Alarm("UNBOUND", "Unbound symbol " + this);
	}

	/*
		Runtime type
	*/
	public Expr type() {
		return forName("Symbol");
	}

	/*
		String representation
	*/
	public String toString() {
		return name;
	}

	/*
		This symbol with a number suffix
	*/
	public Symbol withSuffix(int n) {
		return Symbol.forName(this + "_" + n);
	}	

	/*
		Check e for being a symbol
	*/
	public static Symbol checkSymbol(Expr e, Expr client) throws Alarm {
		if (e instanceof Symbol) return (Symbol) e;
		throw new Alarm("TYPE", client + " expects a symbol argument");
	}	
}

//--------------------------------------------------------------------------------------------------
