	
package expressions;

/*
	BasicOp.java
	Copyright (c) A. Clausing (achim.clausing@uni-muenster.de)

	The basic operators for t.Scheme
*/

import tanagra.Sys;

class Operator_op extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 1);
		Symbol opname = checkSymbol(args.first());
		return Operator.forName(opname.toString());
	}
}

class Operator_bind extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 2);
		Symbol sym = checkSymbol(args.first());
		env.checkUnbound(sym);
		Expr e = args.second().eval(env);
		env.bind(sym, e);
		env.protect(sym);
		return VoidExpr.VOID;
	}
}

class Operator_delete extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		while (args.length > 0) {
			Symbol sym = checkSymbol(args.first());
			env.delete(sym);
			args = args.cdr();
		}
		return VoidExpr.VOID;
	}
}

class Operator_equal extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 2);
		Expr arg1 = args.first().eval(env);
		Expr arg2 = args.second().eval(env);
		return Bool.valueOf(arg1.equals(arg2));
	}
}

class Operator_type extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 1);
		return args.first().eval(env).type();
	}
}

class Operator_let extends Operator {
	public Expr apply(List args, Env env, boolean tailPosition) throws Alarm {
		checkArityodd(args);
		Frame frame = new Frame();
		Env newenv = new Env(frame, env);
		while (args.length() >= 3) {
			Symbol sym = checkSymbol(args.first());
			frame.bind(sym, args.second().eval(newenv));
			args = args.cdr().cdr();
		}
		return args.car().eval(newenv, tailPosition);
	}
}

class Operator_set extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 2);
		Symbol sym = checkSymbol(args.first());
		env.checkProtected(sym);
		Expr e = args.second().eval(env);
		env.bind(sym, e);
		return e;
	}
}

class Operator_seq extends Operator {
	public Expr apply(List args, Env env, boolean tailPosition) throws Alarm {
		// no arity check
		if(args.length() == 0) return VoidExpr.VOID;
		while (!args.cdr().isEmpty()) {
			args.car().eval(env);
			args = args.cdr();
		}
		return args.car().eval(env, tailPosition);
	}
}

class Operator_print extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		// no arity check
		while (!args.isEmpty()) {
			Sys.print(args.car().eval(env));
			args = args.cdr();
		}	
		return VoidExpr.VOID;
	}
}

//------------------------------------------------------------------------------
