	
package expressions;

/*
	ControlOp.java
	Copyright (c) A. Clausing (achim.clausing@uni-muenster.de)

	The operators altering the control flow
*/

class Operator_cond extends Operator {
	public Expr apply(List args, Env env, boolean tailPosition) throws Alarm {
		// no arity check
		while (true) {
			switch (args.length()) {
			case 0: return VoidExpr.VOID;
			case 1:	return args.first().eval(env, tailPosition);
			default:
				boolean cond = checkBool(args.first().eval(env));
				if (cond) {
					return args.second().eval(env, tailPosition);
				} else {
					args = args.cdr().cdr();
					// try next condition-expression pair
				}
			}
		}
	}

	private boolean checkBool(Expr e) throws Alarm {
		return Bool.checkBool(e, this);
	}
}

class Operator_loop extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 1);
		while (true) args.first().eval(env);
		// No return value
	}
}

class Operator_catch extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 3);
		Symbol name = checkSymbol(args.first().eval(env));
		Expr e;
		EvalStack stack = EvalStack.stack();
		try {
			e = args.second().eval(env);
		} catch (Alarm alarm) {
			if (!name.equals(alarm.name())) throw alarm;
			EvalStack.setState(stack);
			Frame frame = new Frame();
			frame.bind(name, alarm);
			env = new Env(frame, env);
			e = args.third().eval(env);
		}
		return e;
	}
}

class Operator_throw extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 1, 2);
		boolean onearg = args.length() == 1;
		Expr arg1 = args.first().eval(env);
		Symbol name;
		Expr value;
		if (onearg && (arg1 instanceof Alarm)) {
			throw (Alarm) arg1; 
		} else if (onearg) {
			name = Symbol.forName("ERROR");
			value = arg1;
		} else {
			name = checkSymbol(arg1);
			value = args.second().eval(env);
		}
		throw new Alarm(name, value);
	}
}

class Operator_eval extends Operator {
	public Expr apply(List args, Env env, boolean tailPosition) throws Alarm {
		checkArity(args, 1, 2);
		Expr expr = args.first().eval(env);
		Env context = (args.length() == 1) ?  env :
			Env.checkEnv(args.second().eval(env), this);
		return expr.eval(context, tailPosition);
	}
}

class Operator_exit extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 0);
		System.exit(0);
		return null;  // To please the compiler...
	}
}

//------------------------------------------------------------------------------
