package expressions;

/*
	FunctionOp.java
	Copyright (c) A. Clausing (achim.clausing@uni-muenster.de)

	Operators for functions and macros
*/

class Operator_function extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 2, 3);
		Symbol name = Symbol.forName("");
		if (args.length() == 3) {
			name = checkSymbol(args.first());
			args = args.cdr();
		}
		Expr body = args.second();
		Expr e = null;
		if (args.first() instanceof Symbol) {
			Symbol sym = (Symbol) args.first();
			e = new Function(sym, body, env, name);
		} else {
			List formals = checkSymlist(args.first());
			e = new Function(formals, body, env, name);
		}
		return e;
	}
}

class Operator_macro extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 2, 3);
		Symbol name = Symbol.forName("");
		if (args.length() == 3) {
			name = checkSymbol(args.first());
			args = args.cdr();
		}
		Expr body = args.second();
		Expr e = null;
		if (args.first() instanceof Symbol) {
			Symbol sym = (Symbol) args.first();
			e = new Macro(sym, body, name);
		} else {
			List formals = checkSymlist(args.first());
			e = new Macro(formals, body, name);
		}
		return e;
	}
}

class Operator_define extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 2);
		List template = checkNonemptysymlist(args.first());
		Symbol sym = (Symbol) template.car();
		List formals = template.cdr();
		List ls = args.cdr().cons(formals).cons(sym);
		Operator functionOp = Operator.forName("function");
		Expr e = functionOp.apply(ls, env);
		env.checkProtected(sym);
		env.bind(sym, e);
		env.protect(sym);
		return e;
	}
}

class Operator_defmacro extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 2);
		List template = checkNonemptysymlist(args.first());
		Symbol sym = (Symbol) template.car();
		List formals = template.cdr();
		List ls = args.cdr().cons(formals).cons(sym);
		Operator functionOp = Operator.forName("macro");
		Expr e = functionOp.apply(ls, env);
		env.checkProtected(sym);
		env.bind(sym, e);
		env.protect(sym);
		return e;
	}
}

//------------------------------------------------------------------------------
