	
package expressions;

/*
	ListOp.java
	Copyright (c) A. Clausing (achim.clausing@uni-muenster.de)

	The base class for the list operators
*/

class Operator_length extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 1);
		return new Num(checkList(args.first().eval(env)).length());
	}
}

class Operator_quote extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 1);
		return args.first();
	}
}

class Operator_cons extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 2);
		Expr car = args.first().eval(env);
		List cdr = checkList(args.second().eval(env));
		return cdr.cons(car);
	}
}

class Operator_car extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 1);
		List ls = checkNonemptylist(args.first().eval(env));
		return ls.car();
	}
}

class Operator_cdr extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 1);
		List ls = checkNonemptylist(args.first().eval(env));
		return ls.cdr();
	}
}

class Operator_setcar extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 2);
		List ls = checkList(args.first().eval(env));
		ls.setHead(args.second().eval(env));
		return VoidExpr.VOID;
	}
}

class Operator_setcdr extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 2);
		List ls = checkList(args.first().eval(env));
		ls.setTail(checkList(args.second().eval(env)));
		return VoidExpr.VOID;
	}
}

//------------------------------------------------------------------------------
