	
package expressions;

import java.math.BigInteger;
import java.util.Arrays;
import tanagra.Sys;

/*
	NumberOp.java
	Copyright (c) A. Clausing (achim.clausing@uni-muenster.de)

	Arithmetic operators for the t.Languages
*/

public class NumberOp extends Operator implements Traceable {

	private boolean tracing = false;

	/*
		Print tracing information for this operator.
	*/	
	protected Expr trace(Num n, Num m, Expr res) {
		String in = "(" + opname() + " " + n + " " + m + ")";
		if (tracing) {
			Sys.println("\tCall \t" + in + "\n\tReturn \t" + res + " from " + in);
		}	
		return res;
	}

	protected Expr trace(Num n, Expr res) {
		String in = "(" + opname() + " " + n + ")";
		if (tracing) {
			Sys.println("\tCall \t" + in + "\n\tReturn \t" + res + " from " + in);
		}	
		return res;
	}

	/*
		Use nicer operator names for tracing
	*/
	private static java.util.List<String> names =
		Arrays.asList("plus", "minus", "mult", "div", "less");
	private static String[] shortnames  =
		{"+",    "-",     "*",    "/",   "<"   };
	private String opname() {
		String opname = name().toString();
		int i = names.indexOf(opname);
		return (i != -1) ? shortnames[i] : opname;
	}

	/*
		Returns the tracing mode of this operator
	*/
	public boolean tracing() {
		return tracing;
	}

	/*
		Toggles tracing mode between on and off.
	*/
	public boolean toggleTracingMode() {
		return tracing = !tracing;
	}
}


class Operator_plus extends NumberOp {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 2);
		Num n = checkNum(args.first().eval(env));
		Num m = checkNum(args.second().eval(env));
		return trace(n, m, n.add(m));
	}
}

class Operator_minus extends NumberOp {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 2);
		Num n = checkNum(args.first().eval(env));
		Num m = checkNum(args.second().eval(env));
		return trace(n, m, n.sub(m));
	}
}

class Operator_mult extends NumberOp {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 2);
		Num n = checkNum(args.first().eval(env));
		Num m = checkNum(args.second().eval(env));
		return trace(n, m, n.mult(m));
	}
}

class Operator_div extends NumberOp {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 2);
		Num n = checkNum(args.first().eval(env));
		Num m = checkNum(args.second().eval(env));
		return trace(n, m, n.div(m));
	}
}

class Operator_less extends NumberOp {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 2);
		Num n = checkNum(args.first().eval(env));
		Num m = checkNum(args.second().eval(env));
		return trace(n, m, Bool.valueOf(n.compareTo(m) == -1));
	}
}

class Operator_round extends NumberOp {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 2);
		Num n = checkNum(args.first().eval(env));
		int digits = checkInt(args.second().eval(env));
		return trace(n, n.round(digits));
	}
}

class Operator_num extends NumberOp {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 1);
		Num n = checkNum(args.first().eval(env));
		return trace(n, new Num(n.numerator()));
	}
}

class Operator_den extends NumberOp {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 1);
		Num n = checkNum(args.first().eval(env));
		return trace(n, new Num(n.denominator()));
	}
}

//------------------------------------------------------------------------------
