	
package expressions;

/*
	ParOp.java
	Copyright (c) A. Clausing (achim.clausing@uni-muenster.de)

	Parallel execution operator
*/

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.LinkedList;
import tanagra.Sys;

class Operator_sleep extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 1);
		Num n = checkNum(args.first().eval(env));
		int m = n.toInt(this) / 10;
		try {
			for (int i = 0; i < m; i++) {
				Sys.checkCtrlC();
				Thread.sleep(10);
			}	
		} catch (InterruptedException ie) {
			// this won't happen
		}
		return VoidExpr.VOID;
	}
}

class Operator_par extends Operator {
	public Expr apply(List args, final Env env) throws Alarm {
		checkAritymin(args, 1);
		LinkedList<Future<Expr>> exprs =
			new LinkedList<Future<Expr>>();
		while (!args.isEmpty()) {
			final Expr e = args.first();
			Callable<Expr> callable =
				new Callable<Expr>() {
					public Expr call() throws Alarm {
						return e.eval(env);
					}
				};
			ExecutorService es = Executors.newSingleThreadExecutor();
			exprs.addFirst(es.submit(callable));
			args = args.cdr();
		}
		List ls = NIL;
		for (Future<Expr> expr: exprs) {
			try {
				ls = ls.cons(expr.get());
			} catch (InterruptedException ie) {
				throw new Alarm("PAR", "System interrupt");
			} catch (ExecutionException e) {
				Throwable t = e.getCause();
				Alarm a = (t instanceof Alarm) ? (Alarm) t :
					new Alarm("PAR", "" + t.getMessage());
				ls = ls.cons(a);
			}
		}
		return ls;
	}
}

//------------------------------------------------------------------------------
