	
package expressions;

/*
	StringOp.java
	Copyright (c) A. Clausing (achim.clausing@uni-muenster.de)

	Operators for string creation and manipulation
*/

class Operator_str extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		// no arity check
		String s = "";
		while (!args.isEmpty()) {
			s += args.car().eval(env).toString();
			args = args.cdr();
		}
		return new StringExpr(s);
	}
}

class Operator_sget extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 2, 3);
		String s = checkString(args.first().eval(env)).toString();
		int i = checkInt(args.second().eval(env));
		checkIndex(0, i, s);
		Expr res;
		if (args.length() == 2) {
			res = new StringExpr("" + s.charAt(i));
		} else {
			int j = checkInt(args.third().eval(env));
			checkIndex(i, j, s);
			res = new StringExpr(s.substring(i, j + 1));
		}
		return res;
	}

	//	Check j for being in [i..s.length())
	protected void checkIndex(int i, int j, String s) throws Alarm {
		if (i <= j && j < s.length()) return;
		String mesg = this + " applied to string \"" + s + "\" expects argument i = "
			+ j + " to satisfy " + i + " <= i < " + s.length();
		throw new Alarm("INDEX", mesg);
	}


}

class Operator_slength extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 1);
		String s = checkString(args.first().eval(env)).toString();
		return new Num(s.length());
	}
}

//------------------------------------------------------------------------------
