
package expressions;

/*
	PascalOp.java
	Copyright (c) A. Clausing (achim.clausing@uni-muenster.de)

	The operators for t.Pascal.
*/

class Operator_var extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 2);
		Symbol sym = checkSymbol(args.first());
		Frame frame = env.first();
		frame.checkProtected(sym);
		Expr e = args.second().eval(env);
		frame.bind(sym, e);
		return e;
	}
}

class Operator_record extends Operator {
	public Expr apply(List args, Env env) throws Alarm {
		checkArity(args, 2);
		Symbol sym = checkSymbol(args.first().eval(env));
		int size = checkInt(args.second().eval(env));
		return new Record(sym, size);
	}
}

class Operator_block extends Operator {
	public Expr apply(List args, Env env, boolean tailPosition) throws Alarm {
		// no arity check
		Frame frame = new Frame();
		Env newenv = new Env(frame, env);
		if(args.length() == 0) return VoidExpr.VOID;
		while (!args.cdr().isEmpty()) {
			args.car().eval(newenv);
			args = args.cdr();
		}
		return args.car().eval(newenv, tailPosition);
	}
}

//------------------------------------------------------------------------------
