package tanagra;

/*
	Sys.java
	Copyright (c) A. Clausing (achim.clausing@uni-muenster.de)
	System parameters for the t.Languages
*/

import sun.misc.Signal;
import sun.misc.SignalHandler;
import expressions.Alarm;
import expressions.EvalStack;

public class Sys {

	/**************************************************************************
	 *
	 * Ctrl-C handling.
	 * (Outcomment this section to disable signal handling.)
	 *
	 **************************************************************************/

	/*
		Signal and handler
	*/	
	private static Signal SIGINT = new Signal("INT");
	private static SignalHandler handler = new SignalHandler() {
		public void handle(Signal sig) {
			if (EvalStack.empty()) { 
				System.exit(0);
			} else {
				ctrlc = true;
			}
		}		
	};

	/*
		ctrlc is true while Ctrl-C was pressed but not yet handled
	*/	
	private static boolean ctrlc = false;

	/*
		Install the handler
	*/	
	static {
		Signal.handle(SIGINT, handler);
	}

	/*
		If Ctrl-C was pressed, reset ctrlc and throw an alarm
	*/
	public static void checkCtrlC() throws Alarm {
		if (ctrlc) {
			ctrlc = false;
			throw new Alarm("CTRLC", "Evaluation interrupted");
		}	
	}	

	/**************************************************************************
	 *
	 * Status of tail call elimination
	 *
	 **************************************************************************/

	private static boolean tailCallElimination = false;

	public static boolean tailCallElimination() {
		return tailCallElimination;
	}	

	public static boolean toggleTailCallElimination() {
		return tailCallElimination = !tailCallElimination;
	}

	/**************************************************************************
	 *
	 * Output methods 
	 *
	 **************************************************************************/

	public static void print(Object obj) {
		System.out.print(obj.toString());
	}

	public static void println(Object obj) {
		System.out.println(obj.toString());
	}
	
	public static void printPrimaryPrompt() {
		System.out.print(Sys.primaryPrompt());
	}

	public static void printSecondaryPrompt() {
		System.out.print(Sys.secondaryPrompt());
	}

	/**************************************************************************
	 *
	 * Line numbering
	 *
	 **************************************************************************/

	private static boolean lineNumberingOn = false;

	private static int LINE_COUNTER = 1;

	public static int lineCounter() {
		return LINE_COUNTER;
	}

	public static void advanceLineCounter() {
		LINE_COUNTER++;
	}
	
	public static void toggleLineNumbering() {
		lineNumberingOn = !lineNumberingOn;
	}

	/**************************************************************************
	 *
	 * Primary and secondary prompts
	 *
	 **************************************************************************/

	public static String primaryPrompt() {
		if (lineNumberingOn) {
			return "---" + LINE_COUNTER + "---> ";
		} else {
			return "-> ";
		}
	}

	public static String secondaryPrompt() {
		return "\t";
	}
}

//--------------------------------------------------------------------------------------------------
